/**
 * \addtogroup Watchdog
 * @{
 */

/*****************************************************************************/
/**
 * \file Watchdog.c
 * Watchdog reset timer routines for the Atmel 8051 and Keil C51.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.0
 * \date 21-04-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#include "Watchdog.h"

/*****************************************************************************
 *
 *                         Public Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                              watchdog_init()
 *
 *****************************************************************************/
void watchdog_init(unsigned char cycles_until_reset)
{
    /*
     * Set lowest 3-bits of watchdog timeout cycle count register.
     */
    WDTPRG = (cycles_until_reset & 0x07);

    /*
     * Restart watchdog reset timer.
     */
    watchdog_restart();

    return;
}

/*****************************************************************************
 *
 *                            watchdog_restart()
 *
 *****************************************************************************/
void watchdog_restart(void)
{
    /*
     * Preserve global interrupt state.
     */
    bit ea_state = EA;

    /*
     * Disable global interrupt, regardless of state.
     */
    EA = 0;

    /*
     * Issue contiguous watchdog restart sequence.
     */
    WDTRST = 0x1E;
    WDTRST = 0xE1;

    /*
     * Restore global interrupt state.
     */
    EA = ea_state;

    return;
}  

/** @} */
