/**
 * \addtogroup Watchdog
 * @{
 */

/*****************************************************************************/
/**
 * \file Watchdog.h
 * Watchdog reset timer routines for the Atmel 8051 and Keil C51.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.0
 * \date 21-04-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#ifndef WATCHDOG_H
#define WATCHDOG_H

#include <t89c51ac2.h>


/*****************************************************************************
 *
 *                               Defined Constants
 *
 *****************************************************************************/

/** 
 * Watchdog reset timeout define for reset after 2^14-1 machine cycles.
 * \hideinitializer
 */
#define WATCHDOG_RESET_AT_16383_CYCLES       0u

/** 
 * Watchdog reset timeout define for reset after 2^15-1 machine cycles.
 * \hideinitializer
 */
#define WATCHDOG_RESET_AT_32767_CYCLES       1u

/** 
 * Watchdog reset timeout define for reset after 2^16-1 machine cycles.
 * \hideinitializer
 */
#define WATCHDOG_RESET_AT_65535_CYCLES       2u

/** 
 * Watchdog reset timeout define for reset after 2^17-1 machine cycles.
 * \hideinitializer
 */
#define WATCHDOG_RESET_AT_131071_CYCLES      3u

/** 
 * Watchdog reset timeout define for reset after 2^18-1 machine cycles.
 * \hideinitializer
 */
#define WATCHDOG_RESET_AT_262143_CYCLES      4u

/** 
 * Watchdog reset timeout define for reset after 2^19-1 machine cycles.
 * \hideinitializer
 */
#define WATCHDOG_RESET_AT_524287_CYCLES      5u

/** 
 * Watchdog reset timeout define for reset after 2^20-1 machine cycles.
 * \hideinitializer
 */
#define WATCHDOG_RESET_AT_1048575_CYCLES     6u

/** 
 * Watchdog reset timeout define for reset after 2^21-1 machine cycles.
 * \hideinitializer
 */
#define WATCHDOG_RESET_AT_2097151_CYCLES     7u


/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

/** 
 * Watchdog timer initialisation function.
 * Initialises and starts the Atmel 8051 watchdog timer hardware.
 * Must be called once at program commencement to select number of machine
 * cycles to count before an unchecked watchdog timer will reset the system.
 * \param cycles_until_reset The \b unsigned \b char selecting number of 
 * elapsed machine cycles before WDT reset. 
 * \return -
 */
void watchdog_init(unsigned char cycles_until_reset);

/** 
 * Watchdog timer reset aversion command.
 * Issue periodically before selected cycle count elapses to
 * prevent watchdog timer timeout and reset of system.
 * \param -
 * \return -
 */
void watchdog_restart(void);

#endif

/** @} */
